This fixes printing a REAL_CST generated from value-numbering
punning some bits to a real which turns out as zero with big
negative exponent.  This causes the loop in real_to_decimal_for_mode to
never terminate.

Bootstrap & regtest running on x86_64-unknown-linux-gnu.

2020-05-14  Richard Biener  <rguent...@suse.de>

        PR middle-end/95118
        * real.c (real_to_decimal_for_mode): Make sure we handle
        a zero with nonzero exponent.

        * gcc.dg/pr95118.c: New testcase.
---
 gcc/real.c                     |  4 ++--
 gcc/testsuite/gcc.dg/pr95118.c | 11 +++++++++++
 2 files changed, 13 insertions(+), 2 deletions(-)
 create mode 100644 gcc/testsuite/gcc.dg/pr95118.c

diff --git a/gcc/testsuite/gcc.dg/pr95118.c b/gcc/testsuite/gcc.dg/pr95118.c
new file mode 100644
index 00000000000..69bc47fd7aa
--- /dev/null
+++ b/gcc/testsuite/gcc.dg/pr95118.c
@@ -0,0 +1,11 @@
+/* { dg-do compile } */
+/* { dg-options "-O2 -fdump-tree-fre" } */
+
+void a();
+void b() {
+    union {
+       int c[4];
+       long double d;
+    } e = {{0, 0, 4}};
+    a(e.d);
+}
diff --git a/gcc/real.c b/gcc/real.c
index 00b23ceb41e..09ec5c08c38 100644
--- a/gcc/real.c
+++ b/gcc/real.c
@@ -1714,8 +1714,8 @@ real_to_decimal_for_mode (char *str, const 
REAL_VALUE_TYPE *r_orig,
 
          do_multiply (&u, &v, ten);
 
-         /* Stop if we're now >= 1.  */
-         if (REAL_EXP (&u) > 0)
+         /* Stop if we're now >= 1 or zero.  */
+         if (REAL_EXP (&u) > 0 || u.cl == rvc_zero)
            break;
 
          v = u;
-- 
2.16.4

Reply via email to